import React from 'react';
import { View, Text, TouchableOpacity, Image } from 'react-native';

import { TCCRegion } from '../../utils/types';
import { styles } from './styles';
import { API_HOST } from 'src/constants';

import MarkIcon from 'assets/icons/mark.svg';

export const TccRegionItem = React.memo(
  ({
    item,
    updateTCC
  }: {
    item: TCCRegion;
    updateTCC: (region: number, visits: 0 | 1) => void;
  }) => {
    const name = item.name.split(/ – | - /);

    return (
      <View style={styles.regionItem}>
        <View style={styles.regionItemHeader}>
          <View style={{ gap: 6, flex: 1 }}>
            <Text style={styles.regionItemName}>{name[0]}</Text>
            <Text style={styles.regionItemSubname}>{name[1]}</Text>
          </View>
          <View style={{ flexDirection: 'row', alignSelf: 'flex-start' }}>
            <Image source={{ uri: `${API_HOST}/img/flags_new/${item.flag}` }} style={styles.flag} />
            {item.flag2 && (
              <Image
                source={{ uri: `${API_HOST}/img/flags_new/${item.flag2}` }}
                style={[styles.flag, { marginLeft: -5 }]}
              />
            )}
          </View>
        </View>

        <View style={styles.divider} />

        <View style={styles.regionItemContent}>
          <View style={styles.btnContainer}>
            <TouchableOpacity
              style={[
                styles.btn,
                item.visited > 0 ? styles.visitedButton : styles.markVisitedButton
              ]}
              onPress={() => updateTCC(item.id, item.visited > 0 ? 0 : 1)}
            >
              {item.visited > 0 ? (
                <View style={styles.visitedContainer}>
                  <MarkIcon width={16} height={16} />
                  <Text style={styles.visitedButtonText}>Visited</Text>
                </View>
              ) : (
                <Text style={[styles.markVisitedButtonText]}>Mark visited</Text>
              )}
            </TouchableOpacity>
          </View>
        </View>
      </View>
    );
  }
);
